#include "execute.h"

char * expand_command(char * cmd)
{
	char * p = cmd;
	static char buffer[4096];
	char * s = buffer;
	char * t;

	while (*p) {
		if (*p == '%') {
			p++;
			switch (*p) {
				case '%':
					*s++ = '%';
					break;
				case 'n':
					t = nook_name;
					while(*s++ = *t++);
				case 'd':
					*s++ = 'd';
					break;
				case 'e':
					*s++ = 'e';
					break;
				default:
					return NULL;
			}
		} else {
			*s = *p;
		}
		p++; s++;
	}
	return buffer;
}

char ** strip_args(char * line) {

	char * s = line;
	
	static char *args[16];
	int argc = 0;
	
	int start = 0;
	int end = 0;
	
	int word = 0;
	int quote = 0;

	while(*s) {

		switch(*s) {
			case '\"':
			case '\'':
				if (quote) {
					args[argc] = (char *)strndup(line + start, end-start);
					argc++;
					start = end;
					quote = 0;
					word = 0;
				} else {
					start = end+1;
					quote = 1;
					word = 1;
				}
				break;
			case ' ':
			case '\t':
				if (word && !quote) {
					args[argc] = (char *)strndup(line + start, end-start);
					argc++;
					start = end;
					word = 0;
				}
				break;
			default:
				if (!word) {
					start = end;
					word = 1;
				}
		}
		s++; end++;
	}

	args[argc] = (char *)strndup(line + start, end-start);
	args[++argc] = NULL;

	return args;
}

int execute_command(char *cmd)
{
	int status;
	int pid;
	char * c = expand_command(cmd);

	if ((pid = fork()) != 0) {
		waitpid(pid, &status, 0);
		writelog(" Command %s returns %d\n", c, status);
	} else {
		char **argv, *envp[3];

		argv = strip_args(c);

		envp[0] = "HOME=/";
		envp[1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
		envp[2] = NULL;

		writelog(" Executing %s %s...\n", argv[0], argv[1]);
		status = execve(argv[0], argv, envp);
		/* in case execve failed */
		writelog(" Execution failed : %d\n", status);
		exit(1);
	}
	return 0;
}

int send_mail_fault(char *dst)
{
	int i;
	int status = 0;
	char cmd_line[1024] = "/bin/mail -s nooks-agent ";
	FILE *mail_pipe;

	strcat(cmd_line, dst);

	if ((mail_pipe = popen(cmd_line, "w")) != NULL) {
		fputs("A fault occured in the nook ", mail_pipe);
		fputs(nook_name, mail_pipe);
		fputs("\nThe following drivers were running in the nook:", mail_pipe);
		for (i=0; i < driver_count; i++) {
			fputs("\n * ", mail_pipe);
			fputs(drivers[i].name, mail_pipe);
			fputs(" (", mail_pipe);
			fputs(drivers[i].interface, mail_pipe);
			fputs(")", mail_pipe);
		}
		fputs("\n\nYour system may or may not recover the nook, depending on the drivers configuration.\n", mail_pipe);
		status = pclose(mail_pipe);
	}

	return status;
}

int send_mail_recover(char *dst)
{
	int i;
	int status = 0;
	char cmd_line[1024] = "/bin/mail -s nooks-agent ";
	FILE *mail_pipe;

	strcat(cmd_line, dst);

	if ((mail_pipe = popen(cmd_line, "w")) != NULL) {
		fputs("A fault had occured in the nook ", mail_pipe);
		fputs(nook_name, mail_pipe);
		fputs("\nThe following drivers where running in the nook:", mail_pipe);
		for (i=0; i < driver_count; i++) {
			fputs("\n * ", mail_pipe);
			fputs(drivers[i].name, mail_pipe);
			fputs(" (", mail_pipe);
			fputs(drivers[i].interface, mail_pipe);
			fputs(")", mail_pipe);
		}
		fputs("\n\nNooks had proceeded successfully the recovery of your system.\n", mail_pipe);
		status = pclose(mail_pipe);
	}

	return status;
}

